Package: CARDspa
Title: Spatially Informed Cell Type Deconvolution for Spatial
        Transcriptomics
Version: 1.4.0
Date: 2025-2-6
Authors@R: 
    c(person(given = "Ying", 
    family = "Ma", 
    email = "ying_ma@brown.edu",
    role = "aut"),
    person(given = "Jing", 
    family = "Fu", 
    email = "jing_fu@brown.edu",
    role = "cre"))
Description: CARD is a reference-based deconvolution method that estimates cell 
    type composition in spatial transcriptomics based on cell type specific 
    expression information obtained from a reference scRNA-seq data. A key 
    feature of CARD is its ability to accommodate spatial correlation in the 
    cell type composition across tissue locations, enabling accurate and 
    spatially informed cell type deconvolution as well as refined spatial map 
    construction. CARD relies on an efficient optimization algorithm for 
    constrained maximum likelihood estimation and is scalable to spatial 
    transcriptomics with tens of thousands of spatial locations and tens of 
    thousands of genes. 
License: GPL-3 + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Depends: R (>= 4.3.0)
Imports: Rcpp (>= 1.0.7),RcppArmadillo, SummarizedExperiment, methods,
        MCMCpack, fields, wrMisc, concaveman, sp, dplyr, sf, Matrix,
        RANN, ggplot2, reshape2, RColorBrewer, S4Vectors, scatterpie,
        grDevices,ggcorrplot, stats, nnls, BiocParallel, NMF,
        spatstat.random, gtools, SingleCellExperiment,
        SpatialExperiment
LazyData: false
biocViews: Spatial, SingleCell, Transcriptomics, Visualization
LinkingTo: Rcpp, RcppArmadillo
Suggests: knitr, rmarkdown, testthat, RcppML, BiocStyle
VignetteBuilder: knitr
URL: https://github.com/YMa-lab/CARDspa
BugReports: https://github.com/YMa-lab/CARDspa/issues
git_url: https://git.bioconductor.org/packages/CARDspa
git_branch: RELEASE_3_23
git_last_commit: 67bc02b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:39:31 UTC; biocbuild
Author: Ying Ma [aut],
  Jing Fu [cre]
Maintainer: Jing Fu <jing_fu@brown.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 04:35:54 UTC; unix
