# Generated by roxygen2: do not edit by hand

export(CARD_deconvolution)
export(CARD_imputation)
export(CARD_refFree)
export(CARD_scmapping)
export(CARD_visualize_Cor)
export(CARD_visualize_gene)
export(CARD_visualize_pie)
export(CARD_visualize_prop)
export(CARD_visualize_prop_2CT)
import(Matrix)
import(RcppArmadillo)
import(SingleCellExperiment)
import(SpatialExperiment)
import(concaveman)
import(ggcorrplot)
import(ggplot2)
import(methods)
import(sp)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(MCMCpack,rdirichlet)
importFrom(RANN,nn2)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,colData)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,'%>%')
importFrom(fields,rdist)
importFrom(grDevices,colorRampPalette)
importFrom(gtools,mixedsort)
importFrom(nnls,nnls)
importFrom(reshape2,melt)
importFrom(scatterpie,geom_scatterpie)
importFrom(sf,st_as_sf)
importFrom(spatstat.random,runifdisc)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,reshape)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(wrMisc,rowGrpMeans)
importMethodsFrom(SummarizedExperiment,"colData<-")
useDynLib(CARDspa)
