#' marker gene list
#'
#' The marker gene list is a list format with each element of the list being the
#' cell type specific gene markers.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(markerList)
#' @name markerList
"markerList"
