#' scRNA-seq count data
#'
#' The scRNA-seq count data must be in the format of matrix or sparseMatrix,
#' while each row represents a gene and each column represents a cell.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(sc_count)
#' @name sc_count
"sc_count"
