#' Spatial transcriptomics count data
#'
#' The spatial transcriptomics count data must be in the format of matrix or
#' sparseMatrix, while each row represents a gene and each column represents a
#' spatial location. The column names of the spatial data can be in the
#' “XcoordxYcoord” (i.e., 10x10) format, but you can also maintain your original
#' spot names, for example, barcode names.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(spatial_count)
#' @name spatial_count
"spatial_count"
