# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' SpatialDeconv function based on Conditional Autoregressive model
#' @param XinputIn The input of normalized spatial data
#' @param UIn The input of cell type specific basis matrix B
#' @param WIn The constructed W weight matrix from Gaussian kernel
#' @param phiIn The phi value
#' @param max_iterIn Maximum iterations
#' @param epsilonIn epsilon for convergence
#' @param initV Initial matrix of cell type compositions V
#' @param initb Initial vector of cell type specific intercept
#' @param initSigma_e2 Initial value of residual variance
#' @param initLambda Initial vector of cell type sepcific scalar.
#'
#' @return A list
CARDfree <- function(XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda) {
    .Call('_CARDspa_CARDfree', PACKAGE = 'CARDspa', XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda)
}

#' SpatialDeconv function based on Conditional Autoregressive model
#' @param XinputIn The input of normalized spatial data
#' @param UIn The input of cell type specific basis matrix B
#' @param WIn The constructed W weight matrix from Gaussian kernel
#' @param phiIn The phi value
#' @param max_iterIn Maximum iterations
#' @param epsilonIn epsilon for convergence
#' @param initV Initial matrix of cell type compositions V
#' @param initb Initial vector of cell type specific intercept
#' @param initSigma_e2 Initial value of residual variance
#' @param initLambda Initial vector of cell type sepcific scalar.
#'
#' @return A list
CARDref <- function(XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda) {
    .Call('_CARDspa_CARDref', PACKAGE = 'CARDspa', XinputIn, UIn, WIn, phiIn, max_iterIn, epsilonIn, initV, initb, initSigma_e2, initLambda)
}

