#' Spatial location data
#'
#' The spatial location data must be in the format of data frame while each row
#' represents a spatial location, the first column represents the x coordinate
#' and the second column represents the y coordinate. The rownames of the
#' spatial location data frame should match exactly with the column names of the
#' spatial_count.
#'
#'
#' @docType data
#' @keywords datasets
#' @usage data(spatial_location)
#' @name spatial_location
"spatial_location"
